"""
Main Portal module
"""

import logging
from flask_migrate import Migrate

import portal.tools.logging

from flask import Flask
from flask_cors import CORS
from flask_jwt_extended import JWTManager
from flask_oauthlib.client import OAuth

from portal import rest
from portal.config import CONFIGURATIONS
from portal.tools.gitlab_client import GitlabFactory
from portal.tools.paths import ROOT_DIR
from flask_sqlalchemy import SQLAlchemy

import os
from storage import Storage

db = SQLAlchemy()
jwt = JWTManager()
oauth = OAuth()
cors = CORS(supports_credentials=True)
storage = Storage()
migrate = Migrate(db=db)
gitlab_factory = GitlabFactory()
log = logging.getLogger(__name__)


def configure_app(app: Flask, env: str = None, ignore_local: bool = False) -> Flask:
    """Configures the Flask app based on production stage.
    Args:
        app(Flask): Flask application
        env(str): Environment for the flask application
        ignore_local(bool): Whether to ignore portal.local.cfg
    """
    config_type = env or os.environ.get('PORTAL_CONFIG_TYPE', 'dev')
    log.debug(f"[INIT] Portal Config Type: {config_type}")
    config_object = CONFIGURATIONS[config_type]
    if config_object is None:
        raise RuntimeError(f"There is no config object for: {config_type}")
    app.config.from_object(config_object)
    ignore_local = ignore_local or env == 'test'
    if not ignore_local:
        app.config.from_pyfile(ROOT_DIR / 'portal.local.cfg', silent=True)
        app.config.from_envvar('PORTAL_CONFIG', silent=True)
    app.config['PORTAL_ENV'] = config_type
    return app


def configure_storage(app: Flask) -> Flask:
    """Configures storage
    Args:
        app(Flask): Flask application
    Returns(Flask): Flask application
    """
    storage_config = dict(
        test_files_dir=app.config.get('PORTAL_STORAGE_TEST_FILES_DIR'),
        submissions_dir=app.config.get('PORTAL_STORAGE_SUBMISSIONS_DIR'),
        workspace_dir=app.config.get('PORTAL_STORAGE_WORKSPACE_DIR')
        )
    storage.init_storage(**storage_config)
    return app


def _init_extension(extension, app: Flask, db: SQLAlchemy = None):
    if db is None:
        extension.init_app(app)
    else:
        extension.init_app(app, db=db)
    return extension


def configure_extensions(app):
    """Configures all extensions
    Args:
        app(Flask): Flask application
    Returns(Flask): Flask application
    """
    _init_extension(db, app)
    _init_extension(migrate, app, db)
    _init_extension(jwt, app)
    _init_extension(oauth, app)
    _init_extension(cors, app)
    _init_extension(gitlab_factory, app)
    return app


def create_app(environment: str = None):
    """Creates and initializes the components used in the application
    Returns(Flask): Flask application instance
    """
    app = Flask('portal')
    # app configuration
    configure_app(app, env=environment)
    configure_storage(app)
    configure_extensions(app)
    rest.register_blueprints(app)
    return app


import portal.database.models
