import logging
from typing import List

from portal.database.models import Course, Group, Role
from portal.service import general
from portal.service.groups import copy_group
from portal.service.projects import copy_project
from portal.service.roles import copy_role

log = logging.getLogger(__name__)


def copy_course(source: Course, target: Course, config: dict) -> Course:
    """Copies course and it's other resources (roles, groups, projects)
    The other resources that should be copied are specified in the config

    Args:
        source(Course): From which course the resources will be copied
        target(Course): To which course the resources will be copied
        config(dict): Configuration to specify which resource should be copied

    Returns(Course): Copied course instance (target)
    """
    if config.get('roles'):
        for role in source.roles:
            copy_role(role, target, with_users=config['roles'])
    if config.get('groups'):
        for group in source.groups:
            copy_group(group, target, with_users=config['groups'])
    if config.get('projects'):
        for project in source.projects:
            copy_project(project, target)

    general.write_entity(target)
    log.info(f"[IMPORT] From {source.id} to:  {target.id}.")

    return target


def delete_course(course: Course):
    """Deletes course

    Args:
        course(Course): Course instance
    """
    log.info(f"[DELETE] Course: {course.id} ({course.codename})")
    general.delete_entity(course)


def __set_course_props(course: Course, data: dict):
    return general.update_entity(course, data, allowed=['name', 'codename'])


def update_course(course: Course, data: dict) -> Course:
    """Updates course

    Args:
        course(Course): Course instance
        data(dict): Dictionary containing data which should be changed

    Returns(Course): Updated course instance
    """
    __set_course_props(course=course, data=data)
    log.info(f"[UPDATE] Course {course.id} ({course.codename}): {course}.")
    return course


def create_course(data: dict) -> Course:
    """Creates new course

    Args:
        data(dict): Course representation in dictionary

    Returns(Course): Course instance
    """
    course = Course()
    __set_course_props(course=course, data=data)
    log.debug(f"[CREATE] Course {course.id}: {course}")
    return course


def update_notes_token(course: Course, token: str) -> Course:
    """Updates notes access token of a course.

    Args:
        course(Course): Course instance for which the token is set
        token(str): The new token

    Returns(Course): Course instance
    """
    course.notes_access_token = token
    general.write_entity(course)
    log.info(f"[UPDATE] Notes access token ({course.codename}) [{course.id}]: {token}")
    return course


def find_all_courses():
    return Course.query.all()


def get_users_filtered(course: Course, groups: List[str], roles: List[str]):
    # log.debug('group ids: ', str(len(groups)))
    # log.debug('role ids: ', str(len(roles)))
    groups_entities = Group.query.filter(Group.id.in_(groups)).all()
    # log.debug('group entities size: ', str(len(groups_entities)))
    roles_entities = Role.query.filter(Role.id.in_(roles)).all()
    # log.debug('role entities size: ', str(len(roles_entities)))
    return course.get_users_filtered(groups_entities, roles_entities)
