import logging
from functools import wraps

from flask import request
from flask_restful import abort
from marshmallow import ValidationError
from sqlalchemy.exc import SQLAlchemyError

from portal.service.errors import PortalAPIError

log = logging.getLogger(__name__)


def error_handler(func):
    """Wraps the method and catches an PortalAPIException and calls abort
    Args:
        func: Wrapped method

    Returns: Wrapper function

    """

    @wraps(func)
    @request_logging
    def _func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except PortalAPIError as ex:
            abort(ex.code, message=ex.message)
        except ValidationError as ex:
            log.warning("[VALID] Validation failed: {ex}")
            abort(400, message=f"Validation failed on: {ex.messages}")
        except SQLAlchemyError as ex:
            log.error(f"[DB] Error: {ex}", ex)
            abort(400, message=f"Database error: {ex}")

    return _func


def request_logging(func):
    """Wraps the rest request endpoint and logs the request
    Args:
        func: Wrapped method

    Returns: Wrapper function

    """

    @wraps(func)
    def _func(*args, **kwargs):
        log.debug(f"[REQUEST] {request.method} ({request.url})")
        return func(*args, **kwargs)
    return _func
